(function() {
    var doc = document;
    var isBgColorChanged = false;
    var STATUS_TIPS_CLASS = "status-tips";
    var loadingInterval = null;

    var isLocalImage = function(nodeTarget) {
        return nodeTarget.className.indexOf("default-img") == -1;
    };

    var getImageStatusTips = function(link) {
        var c = link.children;
        var errTips = null;
        for (var i = 0, len = c.length; i < len; i++) {
            var child = c[i];
            if (child.className == STATUS_TIPS_CLASS) {
                errTips = child;
            }
        }
        return errTips;
    };

    var setText = function(target, text) {
        target.innerHTML = "";
        target.appendChild(doc.createTextNode(text));
    };

    function updateStatusTips(node, text) {
        var tipsNode = getImageStatusTips(node);
        if (!tipsNode) {
            tipsNode = doc.createElement("p");
            tipsNode.className = STATUS_TIPS_CLASS;
            node.appendChild(tipsNode);
        }

        setText(tipsNode, text);
    }

    function imageOnInit(node) {
        updateStatusTips(node, "点击加载图片");
    }

    function imageOnLoading(node) {
        updateStatusTips(node, "图片加载中");
    }

    function imageOnErr(node) {
        updateStatusTips(node, "点击重新加载");
    }

    function replaceImage(nodeTarget, src) {
        if (isLocalImage(nodeTarget)) {
            return;
        }

        if (!src) {
            imageOnErr(nodeTarget);
        } else {
            var img = new Image();

            img.onload = function() {
                nodeTarget.innerHTML = "";
                nodeTarget.className = "";
                nodeTarget.appendChild(img);
                if (isBgColorChanged) {
                    isBgColorChanged = false;
                    doc.body.style.backgroundColor = "#f8fafc";
                } else {
                    isBgColorChanged = true;
                    doc.body.style.backgroundColor = "#f8fafc";
                }
            };
            img.onerror = function() {
                img.onload = null;
                imageOnErr(img.parentNode);
                img.onerror = null;
            };
            img.src = src;
        }
    }

    function location(action, args) {
        args = args || {};
        window.location.href = "wumii://hybrid?info=" + encodeURIComponent(JSON.stringify({"action": action , "args": args}));
    }

    var processImagesArr = [];

    function $$(selector) {
        var arr = [];
        if (!/\.| /.test(selector) && selector.substr(0, 1) == "#") {
            return doc.getElementById(selector.substring(1));
        } else {
            var el = doc.querySelectorAll(selector);
            for (var i = 0, len = el.length; i < len; i++) {
                arr.push(el[i]);
            }
        }
        return arr;
    }

    var likeCount = 0;
    var wrapper = $$("#wrapper");

    function replaceIconImage(src, node) {
        var img = new Image();
        img.onload = function() {
            var cls = node.className;
            if (cls) {
                img.className = cls;
            }
            node.parentNode.insertBefore(img, node);
            node.parentNode.removeChild(node);
        };
        img.src = src;
    }

    var fontSizeStyleNode = null;

    function setFontSize(fontSize) {
        if (fontSizeStyleNode != null) {
            fontSizeStyleNode.parentNode.removeChild(fontSizeStyleNode);
            fontSizeStyleNode = null;
        }

        fontSizeStyleNode = doc.createElement("style");
        var styleStr = "#article, #article sup { font-size: " + fontSize + "px; line-height: " + fontSize * 23 / 16 + "px;}" +
                "#article h1, #article h2, #article h3 { font-size: " + (fontSize + 4) + "px;}" +
                "#article h4, #article h5, #article h6 { font-size: " + (fontSize + 2) + "px;}" +
                "#article p, #article div { text-indent: " + fontSize * 2+ "px;}" +
                "#article .wumii-image-desc { line-height: " + (fontSize + 4) + "px; font-size: " + (fontSize - 2) + "px;}" +
                "blockquote { margin-left: " + (fontSize - 4) / 2 + "px; padding-left: " + (fontSize - 4) / 2 + "px;}";

        fontSizeStyleNode.innerHTML = styleStr;
        doc.head.appendChild(fontSizeStyleNode);
    }

    function registerLinkEvent(isAutoLoadImages) {
        var links = doc.getElementsByTagName("a");
        for (var i = 0, len = links.length; i < len; i++) {
            var link = links[i];
            if (isLocalImage(link)) {
                if (link.parentNode.className == "wumii-ad") {
                    link.addEventListener("click", function(e) {
                        e.preventDefault();
                        if (this.getAttribute("link-type") == "external") {
                            location("clickOnExternalLink", {"url": this.getAttribute("href")});
                        } else {
                            location("clickOnSourceLink", {"url":this.getAttribute("href")});
                        }
                    });
                } else if (link.getElementsByTagName("img").length == 0) {
                    link.addEventListener("click", function(e) {
                        e.preventDefault();
                        if (this.parentNode.className == "view-original") {
                            location("clickOnSourceLink", {"url": this.getAttribute("href")});
                        } else {
                            location("clickOnInternalLink", {"url":this.getAttribute("href")});
                        }
                    });
                } else if (link.parentNode.className == "wumii-image") {
                    link.addEventListener("click", function(e) {
                        e.preventDefault();
                        location("showImageInGallery", {"url":this.getAttribute("href")});
                    });
                }
            } else {
                if (link.parentNode.className == "wumii-image") {
                    processImagesArr.push(link);
                    if (isAutoLoadImages) {
                        imageOnLoading(link);
                    } else {
                        imageOnInit(link);
                    }
                    link.addEventListener("click", function(e) {
                        e.preventDefault();
                        if (!isLocalImage(this)) {
                            imageOnLoading(this);
                            location("loadImage", {"url":this.getAttribute("href")});
                        } else {
                            location("showImageInGallery", {"url":this.getAttribute("href")});
                        }
                    });
                }
            }
        }
    }

    function appendStyleSheet(temp) {
        var style = doc.createElement("link");
        style.href = "css/temp" + temp + ".css";
        style.rel = "stylesheet";
        style.type = "text/css";
        doc.head.appendChild(style);
    }

    function updateItemDetail(data) {
        var numObject = $$(".detail .num");
        for (var i = 0, len = numObject.length; i < len; i++) {
            var node = numObject[i];
            var num = data[node.parentNode.className];
            setText(node, num);
            var nextNode = node.nextSibling;
            if (nextNode && nextNode.nodeType != 1) {
                nextNode = nextNode.nextSibling;
            }

            if (nextNode && nextNode.className == "icon") {
                if (num == 0) {
                    node.parentNode.style.display = "none";
                } else {
                    node.parentNode.style.display = "block";
                }
            }
        }
    }

    function updateLikeState(isLike) {
        updateItemDetail({
            "like": isLike ? ++likeCount : --likeCount
        });
    }

    function updateCommentsCount(commentsCount) {
        updateItemDetail({
            "like" : likeCount
        });
    }

    function updateCountAndDetail(info) {
        likeCount = info.likeCount;
        updateItemDetail({
            "like": likeCount
        });
    }

    function imageResize(img, screenWidth) {
        var imgWidth = img.width,
            imgHeight = img.height;
        var maxWidth = screenWidth,
            maxHeight = img.parentNode.offsetHeight;
        var scaledWidth, scaledHeight;

        var ratioWidth = imgWidth / maxWidth,
            ratioHeight = imgHeight / maxHeight;

        if (ratioWidth > ratioHeight) {
            scaledWidth = maxWidth;
            scaledHeight = ratioWidth * maxHeight;
        } else {
            scaledWidth = ratioHeight * maxWidth;
            scaledHeight = maxHeight;
        }

        var top = (scaledHeight - maxHeight) / 2,
            bottom = top + maxHeight,
            left = (scaledWidth - maxWidth) / 2,
            right = left + maxWidth;

        var style = img.style;
        style.position = "absolute";
        style.left = "-" + left + "px";
        style.top = "-" + top + "px";
        style.width = scaledWidth + "px";
        style.height = scaledHeight + "px";
    }

    function updateHeaderImage(screenWidth) {
        var header = $$("#header");
        var imageNode = doc.createElement("div");
        imageNode.className = "wumii-image";
        var image = new Image();
        image.onload = function() {
            imageResize(this, screenWidth);
        };
        image.src = "images/bg_" + Math.ceil(Math.random() * 3) + ".png";

        imageNode.appendChild(image);
        header.appendChild(imageNode);
    }

    //replace remote images to local images
    function replaceImageSrc(remoteSrc, localSrc) {
        for (var i = 0, len = processImagesArr.length; i < len; i++) {
            var link = processImagesArr[i];
            if (link.getAttribute("href") == remoteSrc) {
                replaceImage(link, localSrc);
            }
        }
    }

    function removeArticleFromSite(isSiteApp) {
        if (isSiteApp) {
            var fromSiteNode = $$("#header .from-site")[0];
            fromSiteNode.parentNode.removeChild(fromSiteNode);
        }
    }

    var isContentFinished = false;
    function setContent(contentInfo) {
        var content = contentInfo.content,
            textSize = contentInfo.textSize,
            isAutoLoadImages = contentInfo.isAutoLoadImages,
            isSiteApp = contentInfo.siteAppItem,
            screenWidth = contentInfo.screenWidth;
        var range = doc.createRange();
        range.selectNodeContents(wrapper);
        var fragment = range.createContextualFragment(content);
        wrapper.appendChild(fragment);

        removeArticleFromSite(isSiteApp);

        var temp = Math.floor(Math.random() * 2);
        appendStyleSheet(temp);

        if (temp == 1) {
            updateHeaderImage(screenWidth);
        }
        updateCountAndDetail(contentInfo);

        registerLinkEvent(isAutoLoadImages);

        setFontSize(textSize);

        $$("#likeItUsers").addEventListener("click", function() {
            location("clickOnUsersLikeIt");
        });

        location("setContentFinish");
        isContentFinished = true;
    }

    function setArticleInfo(info) {
        if (!isContentFinished) {
            setTimeout(arguments.callee, 25);
        } else {
            if (!info.err) {
                updateCountAndDetail(info);
            }
        }
    }

    window["updateLikeState"] = updateLikeState;
    window["updateCommentsCount"] = updateCommentsCount;

    window["replaceImageSrc"] = replaceImageSrc;
    window["setContent"] = setContent;
    window["setArticleInfo"] = setArticleInfo;

    window["setFontSize"] = setFontSize;
})();